package lightcontainer.domains;

import lightcontainer.enumerations.TaskStatus;
import lightcontainer.enumerations.TaskType;
import lightcontainer.protocol.ProtocolWriter;

public class Task {
    // Variables
    private TaskStatus status;
    private ProtocolWriter.ProtocolResult command;
    private String client;
    private String storeDomain;

    public Task(TaskStatus status, ProtocolWriter.ProtocolResult command, String client) {
        this.status = status;
        this.command = command;
        this.client = client;
    }

    public static Task newInstance(ProtocolWriter.ProtocolResult command, String client) {
        Task task = new Task(TaskStatus.PENDING, command, client);
        return task;
    }

    /**
     * Permet de savoir si la réponse est destinée au client
     * @param storeDomain Nom du store back end fournissant la réponse.
     * @return TRUE si le client doit recevoir cette réponse.
     */
    public boolean isResponseOfClient(String storeDomain) {
        return (status == TaskStatus.PROCESSING && this.storeDomain.equals(storeDomain));
    }

    /**
     * Permet de récupérer le login du client associé à la tâche
     * @return Login du client
     */
    public String getClient() {
        return client;
    }

    public ProtocolWriter.ProtocolResult getCommand() {
        return this.command;
    }

    public void setDomain(String storeDomain) {
        this.storeDomain = storeDomain;
        if (storeDomain != null) {
            this.status = TaskStatus.PROCESSING;
        }
    }
}
