package lightcontainer.protocol;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.StringJoiner;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public abstract class ProtocolWriter {
    // Variables
    private final Pattern rulePattern;
    private final String cmdName;

    // Constructor
    protected ProtocolWriter(String cmdName, String pattern) {
        this.rulePattern = Pattern.compile(pattern);
        this.cmdName = cmdName;
    }

    /**
     * Permet de récupérer le nom de la commande.
     * @return Nom de la commande.
     */
    public final String getCmdName() {
        return cmdName;
    }

    public static class ProtocolResult {

        private String command;

        public String getCommand() {
            return command;
        }

        private void setCommand(String command) {
            this.command = command;
        }

        /**
         * Permet d'écrire un fichier sur le réseau. Cad envoyer le contenu d'un fichier sur le réseau.
         * Redéfinissez cette méthode pour l'utiliser.
         * @param writer Buffer à remplir qui sera envoyer via le réseau
         */
        public void write(OutputStream writer) {}
    }

    /**
     * Permet de contruire une commande selon une règle établie.
     * @param data Les données à ajouter dans la commande; L'ordre défini leur position dans la commande
     * @return La commande construite
     */
    public final <T extends ProtocolResult> T execute(String... data) {
        // Concatatène le nom de la commande avec les données (trim), avec un espace entre chaque
        StringBuilder builder = new StringBuilder(this.cmdName);

        for (String param : data)
            builder.append(" " + param);

        String command = builder + "\r\n";
        Matcher ruleMatcher = this.rulePattern.matcher(command); // Vérifie que tout match (cf. Matcher). Si match alors on retourne la commande build, sinon on retourne NULL

        if (ruleMatcher.matches()) {
            ProtocolResult result = onExecuted(data);
            result.setCommand(command);
            return (T) result;
        }

        return null;
    }

    /**
     * Cette méthode est appelée lors de l'exécution de la règle
     */
    protected <T extends ProtocolResult> T onExecuted(String... data) {
        return (T) new ProtocolResult();
    }

}
