package lightcontainer.repository;

import lightcontainer.interfaces.ProtocolRepository;
import lightcontainer.protocol.ProtocolReader;
import lightcontainer.protocol.ProtocolWriter;

import java.util.HashSet;
import java.util.Set;

public class ProtocolRepositoryImpl implements ProtocolRepository {
    private final Set<ProtocolReader> readers = new HashSet<>();
    private final Set<ProtocolWriter> writers = new HashSet<>();

    @Override
    public <T extends ProtocolReader.ProtocolResult> T executeReader(String data) {
        for (ProtocolReader reader : readers) {
            T readerResult = reader.execute(data);
            if (readerResult != null) {
                return readerResult;
            }
        }
        return null;
    }

    @Override
    public <T extends ProtocolWriter.ProtocolResult> T  executeWriter(String cmdName, String... data) {
        for (ProtocolWriter writer : writers) {
            T command;
            if (cmdName.equals(writer.getCmdName()) && (command = writer.execute(data)) != null) {
                return command;
            }
        }
        return null;
    }

    @Override
    public void addReader(ProtocolReader reader) {
        this.readers.add(reader);
    }

    @Override
    public void addWriter(ProtocolWriter writer) {
        this.writers.add(writer);
    }
}
