package lightcontainer.storage;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;

public class Repository {

    /**
     * @param filePath The path where the file must be saved
     * @param adapter  The service that converts Objects to Strings
     */
    static void save(String filePath, Adapter adapter) {
        if (filePath != null) {
            String jsonAppData = adapter.toString();
            try (BufferedWriter bufferedWriter = Files.newBufferedWriter(Paths.get(filePath).toAbsolutePath(), StandardCharsets.UTF_8, StandardOpenOption.CREATE, StandardOpenOption.TRUNCATE_EXISTING)) {
                bufferedWriter.write(jsonAppData);
                bufferedWriter.flush();
            } catch (IOException e) {
                System.out.println("Error while saving configuration file !");
            }
        }
    }

    /**
     * @param filePath The path where the file is stored
     * @param adapter  The service that converts Strings to objects
     * @return
     */
    static AppData load(String filePath, Adapter adapter) {
        String jsonString = readFile(filePath);
        return adapter.fromString(jsonString);
    }

    private static String readFile(String filePath) {
        StringBuilder builder = new StringBuilder();
        try (BufferedReader reader = Files.newBufferedReader(Paths.get(filePath).toAbsolutePath(), StandardCharsets.UTF_8)) {
            while (reader.ready()) {
                builder.append(reader.readLine());
            }
        } catch (IOException e) {
            System.out.println("Error while reading configuration file");
            builder.setLength(0);
        }
        return builder.toString();
    }
}
